@@ -59,9 +59,12 @@ def print_no_instances(service):
     print(f'Check the --include flags for active, failed, and killed instances.')
 
 
-def get_token_on_cluster(cluster, token_name, include_services=False):
+def get_token_on_cluster(cluster, token_name, include_services=False, include_deleted=False):
     """Gets the token with the given name on the given cluster"""
-    token_data, token_etag = get_token(cluster, token_name, include='metadata')
+    include_flags = ['metadata']
+    if include_deleted:
+        include_flags.append('deleted')
+    token_data, token_etag = get_token(cluster, token_name, include=include_flags)
     if token_data:
         data = {'count': 1, 'token': token_data, 'etag': token_etag}
         if include_services:
@@ -72,14 +75,14 @@ def get_token_on_cluster(cluster, token_name, include_services=False):
         return {'count': 0}
 
 
-def query_token(clusters, token, include_services=False):
+def query_token(clusters, token, include_services=False, include_deleted=False):
     """
     Uses query_across_clusters to make the token
     requests in parallel across the given clusters
     """
 
     def submit(cluster, executor):
-        return executor.submit(get_token_on_cluster, cluster, token, include_services)
+        return executor.submit(get_token_on_cluster, cluster, token, include_services, include_deleted)
 
     return query_across_clusters(clusters, submit)
 
@@ -171,18 +174,21 @@ def _get_latest_cluster(clusters, query_result):
     :param clusters: list of local cluster configs from the configuration file
     :param query_result: value from query_token function
     :return: Finds latest token configuration from the query_result. Gets the cluster that is configured in the
-     token description and returns a local cluster who's serverside name matches the one specified in the token.
+     token description and returns a local cluster who's server side name matches the one specified in the token.
      If the token's cluster does not exist in one of the local cluster configurations then an Exception is raised.
     """
     token_descriptions = list(query_result['clusters'].values())
     token_result = max(token_descriptions, key=lambda token: token['token']['last-update-time'])
-    cluster_name_goal = token_result['token']['cluster']
-    provided_cluster_names = []
+    if token_result['token'].get('deleted', False):
+        return None
+    cluster_name_goal = token_result['token']['cluster'].upper()
+    provided_cluster_names = set()
     for c in clusters:
         cluster_settings, _ = http_util.make_data_request(c, lambda: http_util.get(c, '/settings'))
-        cluster_config_name = cluster_settings['cluster-config']['name']
-        provided_cluster_names.append(cluster_config_name)
-        if cluster_name_goal.upper() == cluster_config_name.upper():
+        cluster_config_name = cluster_settings['cluster-config']['name'].upper()
+        cluster_local_config_name = c['name'].upper()
+        provided_cluster_names.add(cluster_config_name)
+        if cluster_name_goal in [cluster_config_name, cluster_local_config_name]:
             return c
     raise Exception(f'The token is configured in cluster {cluster_name_goal}, which is not provided.' +
                     f' The following clusters were provided: {", ".join(provided_cluster_names)}.')
@@ -195,7 +201,7 @@ def get_target_cluster_from_token(clusters, token_name, enforce_cluster):
     :param enforce_cluster: boolean describing if cluster was explicitly specified as an cli argument
     :return: Return the target cluster config for various token operations
     """
-    query_result = query_token(clusters, token_name)
+    query_result = query_token(clusters, token_name, include_deleted=True)
     if query_result["count"] == 0:
         raise Exception('The token does not exist. You must create it first.')
     elif enforce_cluster:
