@@ -1,4 +1,3 @@
-import argparse
 from functools import partial
 
 import requests
@@ -44,7 +43,7 @@ def _update_token(cluster, token_name, existing_token_etag, body):
         return 1, None
 
 
-def check_maintenance(clusters, args, enforce_cluster):
+def check_maintenance(clusters, args, _, enforce_cluster):
     """Checks if a token is in maintenance mode and displays the result. Returns 0 if the token is in maintenance mode
     and returns 1 if the token is NOT in maintenance mode."""
     token_name = args['token']
@@ -54,7 +53,7 @@ def check_maintenance(clusters, args, enforce_cluster):
     return 0 if maintenance_mode_active else 1
 
 
-def start_maintenance(clusters, args, enforce_cluster):
+def start_maintenance(clusters, args, _, enforce_cluster):
     """Sets the token in maintenance mode by updating the token user metadata fields"""
     token_name = args['token']
     timeout_secs = args['timeout']
@@ -91,23 +90,32 @@ def start_maintenance(clusters, args, enforce_cluster):
         return return_code
 
 
-def stop_maintenance(clusters, args, enforce_cluster):
+def stop_maintenance(clusters, args, _, enforce_cluster):
     """Stops maintenance mode for a token by deleting the 'maintenance' user metadata field in the token data"""
     token_name = args['token']
     ping_token = args.pop('ping_token', True)
     timeout = args.pop('timeout', 300)
-    wait_for_ping = True
+    wait_for_ping = args.pop('wait', True)
+    check_maintenance_mode = args.pop('check', False)
     cluster, existing_token_data, existing_token_etag = _get_existing_token_data(clusters, token_name, enforce_cluster)
     maintenance_mode_active = _is_token_in_maintenance_mode(existing_token_data)
-    if not maintenance_mode_active:
+    logging.debug(f'Token {token_name} in {cluster} with etag {existing_token_etag} '
+                  f'is {"" if maintenance_mode_active else "not"} in maintenance mode')
+    if not maintenance_mode_active and check_maintenance_mode:
         raise Exception("Token is not in maintenance mode")
-    json_body = existing_token_data
-    json_body.pop("maintenance")
-    return_code, token_etag = _update_token(cluster, token_name, existing_token_etag, json_body)
+    if maintenance_mode_active:
+        json_body = existing_token_data
+        json_body.pop("maintenance")
+        return_code, token_etag = _update_token(cluster, token_name, existing_token_etag, json_body)
+    else:
+        print(f'Token {terminal.bold(token_name)} does not have maintenance mode activated')
+        return_code, token_etag = 0, existing_token_etag
     if return_code == 0:
         if ping_token:
             if token_etag:
-                success = ping_token_on_cluster(cluster, token_name, timeout, wait_for_ping, token_etag)
+                expected_parameters = {}
+                success = ping_token_on_cluster(cluster, token_name, expected_parameters,
+                                                timeout, wait_for_ping, token_etag)
                 return 0 if success else 1
             else:
                 logging.debug(f'Not pinging token {token_name} in {cluster} as token ETag is missing.')
@@ -127,38 +135,47 @@ def maintenance(parser, clusters, args, _, enforce_cluster):
         parser.print_help()
         return 0
     else:
-        return sub_func(clusters, args, enforce_cluster)
+        return sub_func(clusters, args, _, enforce_cluster)
 
 
 def register_check(add_parser):
     """Registers the maintenance check parser"""
     parser = add_parser('check',
-                        help='Checks if a token is in maintenance mode. Exits with code 0 if the token is in '
+                        help='checks if a token is in maintenance mode. Exits with code 0 if the token is in '
                              'maintenance mode and 1 if the token is not in maintenance mode')
     parser.add_argument('token')
     parser.set_defaults(sub_func=check_maintenance)
 
 
-def register_stop(add_parser):
+def register_stop(command_name, add_parser):
     """Registers the maintenance stop parser"""
-    parser = add_parser('stop', help='Stop maintenance mode for a token. '
-                                     'Requests to the token will be handled normally.'
-                                     'By default, also ping the token to ensure a running service.')
+    parser = add_parser(command_name,
+                        help='stop maintenance mode for a token. '
+                             'Requests to the token will be handled normally. '
+                             'By default, also ping the token to ensure a running service.')
     ping_group = parser.add_mutually_exclusive_group(required=False)
     ping_group.add_argument('--no-ping', action='store_false', dest='ping_token',
-                            help='Skip pinging the token. Pinging the token is enabled by default.')
+                            help='skips pinging the token; pinging the token is enabled by default')
     ping_group.add_argument('--ping', action='store_true', dest='ping_token',
-                            help='Ping the token after stopping maintenance.')
-    parser.add_argument('--timeout', '-t', default=300, help='read timeout (in seconds) for ping request.',
+                            help='pings the token after stopping maintenance.')
+    parser.add_argument('--timeout', '-t', default=300, help='read timeout (in seconds) for ping request',
                         type=check_positive)
+    parser.add_argument('--no-wait', '-n', help='do not wait for ping request to return',
+                        dest='wait', action='store_false')
+    check_group = parser.add_mutually_exclusive_group(required=False)
+    check_group.add_argument('--check', action='store_true', dest='check',
+                             help='enforces the check that the token is currently in maintenance mode')
+    check_group.add_argument('--no-check', action='store_false', dest='check',
+                             help='skips checking maintenance mode; skipping the check is enabled by default')
     parser.add_argument('token')
     parser.set_defaults(sub_func=stop_maintenance)
+    return stop_maintenance
 
 
-def register_start(add_parser):
+def register_start(command_name, add_parser):
     """Registers the maintenance start parser"""
-    parser = add_parser('start',
-                        help='Start maintenance mode for a token. '
+    parser = add_parser(command_name,
+                        help='start maintenance mode for a token. '
                              'All requests to this token will begin to receive a 503 response. '
                              "By default, also kill the token's currently running services.")
     kill_group = parser.add_mutually_exclusive_group(required=False)
@@ -176,6 +193,7 @@ def register_start(add_parser):
                         help='Your message will be provided in a 503 response for requests to the token. '
                              'The message cannot be more than 512 characters.')
     parser.set_defaults(sub_func=start_maintenance)
+    return start_maintenance
 
 
 def register(add_parser):
@@ -185,6 +203,6 @@ def register(add_parser):
                         description='Manage maintenance mode for a Waiter token.')
     subparsers = parser.add_subparsers()
     register_check(subparsers.add_parser)
-    register_start(subparsers.add_parser)
-    register_stop(subparsers.add_parser)
+    register_start('start', subparsers.add_parser)
+    register_stop('stop', subparsers.add_parser)
     return partial(maintenance, parser)
