@@ -12,6 +12,17 @@ TRUE_STRINGS = ('yes', 'true', 'y')
 FALSE_STRINGS = ('no', 'false', 'n')
 
 
+def get_in(obj, keys, default_value=None):
+    """Given a list of keys and a value, return the nested value in the object or the default"""
+    cur_node = obj
+    for key in keys:
+        if key in cur_node:
+            cur_node = cur_node[key]
+        else:
+            return default_value
+    return cur_node
+
+
 def update_in(obj, keys, value):
     """Given a list of keys and a value, return a new object with that value set"""
     cur_node = obj
@@ -64,14 +75,14 @@ def str2bool(v):
         return None
 
 
-def response_message(resp_json):
+def response_message(resp_json, default_message='Encountered unexpected error.'):
     """Pulls the error message out of a Waiter response"""
     if 'waiter-error' in resp_json and 'message' in resp_json['waiter-error']:
         message = resp_json['waiter-error']['message']
         if not message.endswith('.'):
             message = f'{message}.'
     else:
-        message = 'Encountered unexpected error.'
+        message = default_message
     return message
 
 
